import random
import string
import uuid
from datetime import datetime, timedelta

import pytest

from datahub_classify.helper_classes import ColumnInfo, Metadata
from datahub_classify.infotype_predictor import predict_infotypes
from datahub_classify.reference_input import input1 as default_config


def random_vehicle_number():
    state_codes = ["MH", "TN", "BH", "DL"]
    separators = ["-", " ", "", "_"]
    return "".join(
        [
            random.choice(state_codes),
            random.choice(separators),
            str(random.randint(1, 20)),
            random.choice(separators),
            "".join(random.choices(string.ascii_letters, k=random.randint(1, 3))),
            random.choice(separators),
            "".join(random.choices(string.digits, k=4)),
        ]
    )


@pytest.fixture(scope="module")
def column_infos():
    return [
        ColumnInfo(
            metadata=Metadata(
                meta_info={
                    "Name": "id",
                    "Description": "Primary",
                    "Datatype": "str",
                    "Dataset_Name": "entry_register",
                }
            ),
            values=[uuid.uuid4() for i in range(1, 100)],
        ),
        ColumnInfo(
            metadata=Metadata(
                meta_info={
                    "Name": "vehicle_number",
                    "Description": "Vehicle registration number ",
                    "Datatype": "str",
                    "Dataset_Name": "entry_register",
                }
            ),
            values=[random_vehicle_number() for i in range(1, 100)],
        ),
        ColumnInfo(
            metadata=Metadata(
                meta_info={
                    "Name": "entry_time",
                    "Description": "Time of vehicle's entry",
                    "Datatype": "datetime",
                    "Dataset_Name": "entry_register",
                }
            ),
            values=[
                datetime.now() - timedelta(hours=random.randint(0, 24))
                for i in range(1, 100)
            ],
        ),
    ]


@pytest.fixture
def custom_config_patch():
    return {
        "IN_Vehicle_Registration_Number": {
            "Prediction_Factors_and_Weights": {
                "Name": 0.2,
                "Description": 0.1,
                "Datatype": 0.1,
                "Values": 0.6,
            },
            "Name": {
                "regex": [
                    "^.*vehicle.*num.*$",
                    "^.*license.*plat.*num.*$",
                    "^.*license.*plat.*num.*$",
                    "^.*vehicle.*plat.*num.*$",
                    "^.*vehicle.*num.*plat.*$",
                ]
            },
            "Description": {
                "regex": [
                    "^.*vehicle.*num.*$",
                    "^.*license.*plat.*num.*$",
                    "^.*license.*plat.*num.*$",
                    "^.*vehicle.*plat.*num.*$",
                    "^.*vehicle.*num.*plat.*$",
                ]
            },
            "Datatype": {"type": ["str", "varchar", "text"]},
            "Values": {
                "prediction_type": "regex",
                "regex": [r"[a-z]{2}[-_\s]?[0-9]{1,2}[-_\s]?[a-z]{2,3}[-_\s]?[0-9]{4}"],
                "library": [],
            },
        }
    }


def test_custom_infotype_prediction(column_infos, custom_config_patch):
    # Default config
    out_column_infos = predict_infotypes(
        column_infos, confidence_level_threshold=0.7, global_config=default_config
    )
    assert not out_column_infos[0].infotype_proposals
    assert not out_column_infos[1].infotype_proposals
    assert not out_column_infos[2].infotype_proposals

    # Config with new custom infotype, all factors
    config_new = default_config.copy()
    config_new.update(custom_config_patch)
    out_column_infos = predict_infotypes(
        column_infos, confidence_level_threshold=0.7, global_config=config_new
    )
    assert not out_column_infos[0].infotype_proposals
    assert not out_column_infos[2].infotype_proposals

    predicted_infotypes = out_column_infos[1].infotype_proposals
    assert predicted_infotypes
    assert len(predicted_infotypes) == 1
    assert predicted_infotypes[0].infotype == "IN_Vehicle_Registration_Number"
    assert predicted_infotypes[0].debug_info.name == 1
    assert predicted_infotypes[0].debug_info.description == 1
    assert predicted_infotypes[0].debug_info.datatype == 1



from exclude_name_test_config import (
    exclude_name_test_config,
    none_exclude_name_test_config,
)

from datahub_classify.helper_classes import ColumnInfo, Metadata
from datahub_classify.infotype_utils import perform_basic_checks, strip_formatting


def column_infos():
    return [
        ColumnInfo(
            metadata=Metadata(
                meta_info={
                    "Name": "id",
                    "Description": "Unique identifier",
                    "Datatype": "int",
                    "Dataset_Name": "email_data",
                }
            ),
            values=[1, 2, 3, 4, 5, 6, 7, 8, 9, 10],
        ),
        ColumnInfo(
            metadata=Metadata(
                meta_info={
                    "Name": "email_from",
                    "Description": "Sender's email address",
                    "Datatype": "str",
                    "Dataset_Name": "email_data",
                }
            ),
            values=[
                "bob.myers@gmail.com",
                "jeremy.strong@nyu.edu",
                "alice.smith@yahoo.com",
                "maria.lopez@outlook.com",
                "lucas.grant@edu.org",
                "nora.jones@live.com",
                "ethan.knight@techhub.com",
                "chloe.wilson@health.org",
                "leonard.hayes@finance.com",
                "sophie.turner@arts.com",
            ],
        ),
        ColumnInfo(
            metadata=Metadata(
                meta_info={
                    "Name": "email_to",
                    "Description": "Recipient's email address",
                    "Datatype": "str",
                    "Dataset_Name": "email_data",
                }
            ),
            values=[
                "susan.bones@msn.com",
                "craig.chatterson@datahub.io",
                "john.doe@example.com",
                "kevin.miller@company.com",
                "emily.harris@webmail.com",
                "philip.watson@service.net",
                "diana.ross@music.com",
                "brad.pitts@cinema.com",
                "grace.adams@legalteam.org",
                "rick.martin@sports.net",
            ],
        ),
        ColumnInfo(
            metadata=Metadata(
                meta_info={
                    "Name": "email_sent",
                    "Description": "Indicates if email was sent",
                    "Datatype": "bool",
                    "Dataset_Name": "email_data",
                }
            ),
            values=[False, True, True, False, True, False, True, True, False, True],
        ),
        ColumnInfo(
            metadata=Metadata(
                meta_info={
                    "Name": "email_received",
                    "Description": "Indicates if email was received",
                    "Datatype": "bool",
                    "Dataset_Name": "email_data",
                }
            ),
            values=[False, True, False, False, True, False, False, True, False, False],
        ),
    ]


def test_perform_basic_checks_with_exclude_name():
    for col_data in column_infos():
        result = perform_basic_checks(
            Metadata(col_data.metadata.meta_info),
            col_data.values,
            exclude_name_test_config["Email_Address"],
            "Email_Address",
            1,
        )
        if col_data.metadata.meta_info["Name"] in ["email_sent", "email_received"]:
            assert not result
        else:
            assert result


def test_perform_basic_checks_with_none_exclude_name():
    for col_data in column_infos():
        result = perform_basic_checks(
            Metadata(col_data.metadata.meta_info),
            col_data.values,
            none_exclude_name_test_config["Email_Address"],
            "Email_Address",
            1,
        )
        assert result


def test_strip_formatting():
    assert strip_formatting("Name") == "name"
    assert strip_formatting("my_column_name") == "mycolumnname"
    assert strip_formatting("Col.Name") == "colname"
